// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// backgroundPage.js
//  - Manages state over the whole extension (e.g. badge blink)
//  - Offers connection from content scripts & conceptboard.com (chrome.runtime.sendMessage)
//  - defines cb.background
//  - depends on cb.util

var cb = cb ? cb : {};
cb.background = {

    _badgeInterval  : null,
    _badgeText      : '',

    setBadgeText : function(text) {
        cb.background._badgeText = text;
        chrome.browserAction.setBadgeText({ text : cb.background._badgeText });
    },

    startBadgeBlink : function() {
        if (cb.background._badgeInterval) {
            clearInterval(cb.background._badgeInterval);
        }
        cb.background._badgeInterval = window.setInterval(function() {
            cb.background.setBadgeText(cb.background._badgeText === '' ? '1' : '');
        }, 500);
        cb.background.setBadgeText('1');
        chrome.browserAction.setBadgeBackgroundColor({ color:'#ff6f68' });
    },

    stopBadgeBlink : function() {
        if (cb.background._badgeInterval) {
            clearInterval(cb.background._badgeInterval);
            cb.background._badgeInterval = null;
        }
        cb.background.setBadgeText('');
    }

};

// Communication with content scripts
chrome.runtime.onMessage.addListener(function(request, sender, callback) {
    if (request.type === "capture") {
        cb.util.captureScreen(function(img, imgWidth, imgHeight) {
            var scale = request.innerWidth / imgWidth;
            var dx = request.left * -1;
            var dy = request.top * -1;
            var dw = imgWidth * scale;
            var dh = imgHeight * scale;
            var canvas = document.createElement('canvas');
            canvas.width = request.width;
            canvas.height = request.height;
            canvas.getContext("2d").drawImage(img, dx, dy, dw, dh);
            var dataUrl = canvas.toDataURL();
            cb.util.storeDataUrl(dataUrl, sender.url, 'clip', function() {
                cb.background.startBadgeBlink();
                callback();
            });
        });
    } else if (request.type === "main") {
        cb.background.stopBadgeBlink();
    }
    // in any case wait for an async response
    return true;
});

// Communication with *.conceptboard.com
chrome.runtime.onMessageExternal.addListener(function(request, sender, sendResponse) {
    if (request.ping){
        sendResponse({pong: true});
    }
});
