// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// lib-performance.js
//  - helper functions to use in frontend ONLY
//  - defines cb.performance
//  - no dependencies
//
// Notes on extension performance:
//  - After some (unknown) initialization, between loading the first JS and having the page ready, not much time is needed anymore.
//  - We can assume, that after extension packaging, the initial time is decreased; anyways we should not add much additional time on top.
//  - Some HTML without images and without any other CSS or JS (expect this) loads about 20ms (DOMContentLoaded after pageLoadStart).
//  - Loading the 5 tips images directly in the page increases the time only between 5 and 10ms.
//  - Loading all CSS and JS (including jQuery), but without executing any JS increases the whole time to about 200-250ms.
//  - Executing all synchronous JS (connecting events, adding fixed DOM objects etc.) brings the time to 270-350ms.
//  - Accessing an empty extension storage (no matter for size, actual content or both) brings the time from to 500-600ms.
//    As the time difference is not related to the actual number or type of operation, perhaps only the first call is slow!
//  - If there's 1 full page screenshot of 5MB, the storage access takes 2000ms (but that is the actual image display only).
//    If we're loading the image and do everything BUT actual creating the image tag, we're back at the original 500-600ms!
//  - Therefor we shall NOT render all stored images when loading!

var cb = cb ? cb : {};
cb.performance = {
	stamp : new Date().getTime(),
	log : function(debugMsg) {
	  console.debug('Performance log:', debugMsg, (new Date().getTime() - cb.performance.stamp) + 'ms');
	},
	reset : function(debugMsg) {
	  cb.performance.stamp = new Date().getTime();
	  console.debug('Performance reset:', debugMsg, new Date());
	}
};
document.addEventListener('DOMContentLoaded', function() {
  	cb.performance.log('DOM content loaded');
});
console.debug('Performance log: Init at', new Date());
