// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// lib-storage.js
//  - helper functions for storage to use in frontend ONLY
//  - defines cb.storage
//  - depends on cb.config


var cb = cb ? cb : {};
cb.storage = {

  /** Sync the global config/defaultConfig objects with the local storage. */
  syncConfigWithStore : function() {
    cb.config = cb.storage.getStoredConfig();
  },

  /** Returns the saved config object. Store and returns the existing global 'config' object if none present. */
  getStoredConfig : function () {
    var storedConfig = cb.storage._getLocalStorageObject('CBExt-Config');
    if (!storedConfig) {
      storedConfig = cb.storage._setLocalStorageObject('CBExt-Config', cb.defaultConfig);
    }
    return storedConfig;
  },

  /** Updates a key calue pair within the config, stores it and updates cb.config */
  storeConfigUpdate : function(key, value) {
    var storedConfig = cb.storage.getStoredConfig();
    storedConfig[key] = value;
    cb.config = cb.storage._setLocalStorageObject('CBExt-Config', storedConfig);
  },

  /** Clears everything in the extension storage AND the config in local storage. */
  clearAllStorage : function(callback) {
    chrome.storage.local.clear(function() {
      cb.storage._deleteLocalStorageObject('CBExt-Config');
      cb.storage.syncConfigWithStore();
      callback();
    });
  },

  /** Clears specified item from the storage **/
  clearItemFromStorage : function (key, callback){
    chrome.storage.local.remove(key, function(){
        callback();
    });
  },

  /** Retrieves all data urls in the extension storage items form (starting with the oldest). */
  retrieveStoredDataUrls : function(callback) {
    chrome.storage.local.get(function(items) { 
      callback(items);
    });
  },

  /** Retrieves the used space in the extension storage. */
  retrieveUsedDataUrlStorageInBytes : function(callback) {
    chrome.storage.local.getBytesInUse(function(bytesInUse) { callback(bytesInUse); });
  },

  /** Convert a data URL to a blob */
  dataUrlToBlob : function(dataUrl){
    // see http://stackoverflow.com/a/17332272/150955
    var data = atob(dataUrl.substring("data:image/png;base64,".length));
    var asArray = new Uint8Array(data.length);
    for( var i = 0, len = data.length; i < len; ++i ) {
        asArray[i] = data.charCodeAt(i);    
    }
    var blob = new Blob([ asArray.buffer ], {type: "image/png"});
    return window.URL.createObjectURL(blob);
  },

  /** Reads from local storage. Returns the object read or null. */
  _getLocalStorageObject : function(key) {
    try {
      return JSON.parse(localStorage.getItem(key));
    } catch (e) {
      console.error("Error reading from local storage: ", key, e);
    }
  },

  /** Stores objects in local storage. Returns the saved object or null. */
  _setLocalStorageObject : function(key, obj) {
    try {
      localStorage.setItem(key, JSON.stringify(obj));
      return obj;
    } catch (e) {
      console.error("Error writing to local storage. ", key, obj, e);
    }
  },

  /** Delete from local storage. Returns true if succesful. */
  _deleteLocalStorageObject : function (key) {
    try {
      localStorage.removeItem(key);
      return true;
    } catch (e) {
      console.error("Error deleting from local storage. ", key, e);
    }
  }

};
